require 'digest'
require 'json'
require 'rox/core/consts/property_type'

module Rox
  module Core
    class BUID
      BUID_GENERATORS = [
        PropertyType::PLATFORM,
        PropertyType::APP_KEY,
        PropertyType::LIB_VERSION,
        PropertyType::API_VERSION
      ].freeze

      def initialize(sdk_settings, device_properties, _flag_repository, _custom_property_repository)
        @sdk_settings = sdk_settings
        @device_properties = device_properties
        @buid = nil
      end

      def value
        properties = @device_properties.all_properties
        values = []
        BUID::BUID_GENERATORS.each do |pt, _|
          values << properties[pt.name] if properties.include?(pt.name)
        end

        hash = Digest::MD5.hexdigest(values.join('|'))

        @buid = hash.upcase
      end

      def query_string_parts
        generators = BUID::BUID_GENERATORS.map { |pt, _| pt.name }

        {
          PropertyType::BUID.name => value,
          PropertyType::BUID_GENERATORS_LIST.name => generators.join(',')
        }
      end

      def to_s
        @buid
      end
    end
  end
end
