require 'rox/core/consts/property_type'
require 'rox/core/consts/build'

module Rox
  module Core
    class DeviceProperties
      attr_reader :rox_options

      def initialize(sdk_settings, rox_options)
        @sdk_settings = sdk_settings
        @rox_options = rox_options
      end

      def get(property)
        all_properties[property.name]
      end

      def all_properties
        {
          PropertyType::PACKAGE_NAME.name => @rox_options.version,
          PropertyType::VERSION_NAME.name => @rox_options.version,
          PropertyType::LIB_VERSION.name => lib_version,
          PropertyType::ROLLOUT_BUILD.name => '50',
          PropertyType::API_VERSION.name => Build::API_VERSION,
          PropertyType::APP_VERSION.name => @rox_options.version,
          PropertyType::APP_RELEASE.name => @rox_options.version,
          PropertyType::DISTINCT_ID.name => distinct_id,
          PropertyType::APP_KEY.name => @sdk_settings.api_key,
          PropertyType::PLATFORM.name => Build::PLATFORM
        }
      end

      def rollout_environment
        env = ENV['ROLLOUT_MODE']
        %w[QA LOCAL].include?(env) ? env : 'PRODUCTION'
      end

      def lib_version
        '1.0.0'
      end

      def rollout_key
        all_properties[PropertyType::APP_KEY.name]
      end

      def distinct_id
        'stam'
      end
    end
  end
end
