require 'rox/core/configuration/configuration_fetched_args'

module Rox
  module Core
    class ConfigurationFetchedInvoker
      def initialize(user_unhandled_error_invoker)
        @fetched_handlers = []
        @mutex = Mutex.new
        @user_unhandled_error_invoker = user_unhandled_error_invoker
      end

      def invoke(fetcher_status, creation_date, has_changes)
        raise_fetched_event(Rox::Core::ConfigurationFetchedArgs.new(fetcher_status, creation_date, has_changes))
      end

      def invoke_error(error_details)
        raise_fetched_event(Rox::Core::ConfigurationFetchedArgs.error(error_details))
      end

      def register_fetched_handler(&block)
        @mutex.synchronize do
          @fetched_handlers << block
        end
      end

      def raise_fetched_event(args)
        handlers = []
        @mutex.synchronize do
          handlers = @fetched_handlers.clone
        end

        handlers.each do |handler|
          begin
            handler.call(args)
          rescue StandardError => e
            user_unhandled_error_invoker.invoke(handler, Rox::Core::CONFIGURATION_FETCHED_HANDLER, e)
          end
        end
      end
    end
  end
end
