require 'rox/core/entities/rox_string'
require 'rox/core/entities/default_flag_values'

module Rox
  module Core
    class Flag < RoxString
      FLAG_TRUE_VALUE = 'true'.freeze
      FLAG_FALSE_VALUE = 'false'.freeze

      def initialize(default_value = DefaultFlagValues::BOOLEAN)
        super(default_value ? Flag::FLAG_TRUE_VALUE : Flag::FLAG_FALSE_VALUE, [Flag::FLAG_FALSE_VALUE,
                                                                               Flag::FLAG_TRUE_VALUE])
      end

      def enabled?(context)
        merged_context = MergedContext.new(@parser&.global_context, context)
        value = internal_enabled?(merged_context)
        if [true, false].include? value
          send_impressions(value, merged_context)
          return value
        end

        send_impressions(DefaultFlagValues::BOOLEAN, merged_context)
        DefaultFlagValues.BOOLEAN
      end

      def value(context = nil)
        merged_context = MergedContext.new(@parser&.global_context, context)
        internal_value(merged_context, false)
      end

      def internal_enabled?(context, nil_instead_of_default = false)
        val = internal_value(context, nil_instead_of_default)
        nil_instead_of_default && val.nil? ? nil : (val == Flag::FLAG_TRUE_VALUE)
      end

      def enabled(context)
        yield if enabled?(context)
      end

      def disabled(context)
        yield unless enabled?(context)
      end
    end
  end
end
