require 'rox/core/logging/logging'

module Rox
  module Core
    class UserspaceUnhandledErrorInvoker
      def initialize(user_unhandler_error_handler = nil)
        @user_unhandler_error_handler = user_unhandler_error_handler
      end

      def invoke(exception_source, exception_trigger, exception)
        unless @user_unhandler_error_handler
          Logging.logger.error("User Unhandled Error Occurred, no fallback handler was set, exception ignored: #{exception}")
          return
        end

        begin
          userspace_unhandled_error_args = UserspaceUnhandledErrorArgs.new(exception_source, exception_trigger, exception)
          @user_unhandler_error_handler.call(userspace_unhandled_error_args)
        rescue StandardError => e
          Logging.logger.error("User Unhandled Error Handler itself threw an exception. original exception: #{e}")
        end
      end

      def handler=(handler)
        @user_unhandler_error_handler = handler
      end
    end

    class UserspaceUnhandledErrorArgs
      def initialize(exception_source = nil, exception_trigger = nil, exception = nil)
        @exception_source = exception_source
        @exception_trigger = exception_trigger
        @exception = exception
      end

      def to_s
        "UserspaceUnhandledErrorArgs(#{@exception_source}, #{@exception_trigger}, #{@exception})"
      end
    end
  end
end
