require 'rox/core/configuration/fetcher_error'
require 'rox/core/logging/logging'

module Rox
  module Core
    class ConfigurationFetcherBase
      def initialize(request_configuration_builder, request, configuration_fetched_invoker)
        @request_configuration_builder = request_configuration_builder
        @request = request
        @configuration_fetched_invoker = configuration_fetched_invoker
      end

      def write_fetch_error_to_log_and_invoke_fetch_handler(source, response, raise_configuration_handler = true, next_source = nil)
        retry_msg = next_source.nil? ? '' : "Trying from #{next_source}. "
        Logging.logger.debug("Failed to fetch from #{source}. #{retry_msg}http error code: #{response.status_code}")
        @configuration_fetched_invoker.invoke_error(FetcherError::NETWORK_ERROR) if raise_configuration_handler
      end

      def write_fetch_exception_to_log_and_invoke_fetch_handler(source, ex)
        Logging.logger.error("Failed to fetch configuration. Source: #{source}. Ex: #{ex}")
        @configuration_fetched_invoker.invoke_error(FetcherError::NETWORK_ERROR)
      end
    end
  end
end
