require 'rox/core/network/request_data'
require 'rox/core/consts/environment'
require 'rox/core/consts/property_type'

module Rox
  module Core
    class RequestConfigurationBuilder
      def initialize(sdk_settings, buid, device_properties)
        @sdk_settings = sdk_settings
        @buid = buid
        @device_properties = device_properties
        @rox_options = device_properties.rox_options
      end

      def build_for_roxy
        roxy_endpoint = URI.join(@rox_options.roxy_url, Rox::Core::Environment.roxy_internal_path).to_s
        build_request_with_full_params(roxy_endpoint)
      end

      def build_for_cdn
        RequestData.new("#{Rox::Core::Environment.cdn_path}/#{relative_path}",
                        Rox::Core::PropertyType::DISTINCT_ID.name => @device_properties.distinct_id)
      end

      def build_for_api
        build_request_with_full_params("#{Rox::Core::Environment.api_path(@rox_options.self_managed_options&.server_url)}/#{relative_path}")
      end

      def relative_path
        "#{@device_properties.rollout_key}/#{@buid.value}"
      end

      def build_request_with_full_params(url)
        query_params = {}

        @buid.query_string_parts.each do |key, value|
          query_params[key] = value unless query_params.include?(key)
        end

        @device_properties.all_properties.each do |key, value|
          query_params[key] = value unless query_params.include?(key)
        end

        query_params[Rox::Core::PropertyType::CACHE_MISS_RELATIVE_URL.name] = relative_path
        query_params['devModeSecret'] = @sdk_settings.dev_mode_secret

        RequestData.new(url, query_params)
      end
    end
  end
end
