require 'json'
require 'rox/core/logging/logging'

module Rox
  module Core
    class Response
      attr_accessor :status_code, :text, :content_type

      def initialize(status_code, text, content_type = 'application/octet-stream')
        @status_code = status_code
        @text = text
        @content_type = content_type
      end

      def success?
        status_code_success? && !missing_via_response_body?
      end

      def status_code_success?
        @status_code >= 200 && @status_code < 300
      end

      def missing_via_response_body?
        @status_code == 200 && @content_type == 'application/json' &&
          [404, '404'].include?(JSON.parse(text)['result'])
      rescue JSON::ParserError
        Logging.logger.error("Failed to parse JSON response: #{text}")
        false
      end
    end
  end
end
