require 'em-eventsource'

module Rox
  module Core
    class NotificationListener
      def initialize(listen_url, app_key)
        @listen_url = listen_url
        @app_key = app_key
        @handlers = {}
      end

      def on(event_name, &handler)
        @handlers[event_name] ||= []
        @handlers[event_name] << handler
      end

      def start
        sse_url = "#{@listen_url.chomp('/')}/#{@app_key}"
        @thread = Thread.new do
          EM.run do
            source = EventMachine::EventSource.new(sse_url)
            @handlers.each do |event_name, event_handlers|
              event_handlers.each do |handler|
                source.on event_name do |data|
                  # Start new thread to allow the handler to stop the Listener (terminate the current thread)
                  # and continue handler code execution without interruption
                  handler_thread = Thread.new do
                    handler.call(data)
                  end
                  handler_thread.join
                end
              end
            end
            source.start
          end
        end
      end

      def stop
        @thread&.terminate
        @thread = nil
      end
    end
  end
end
