require 'securerandom'
require 'rox/server/rox_options'
require 'rox/server/client/sdk_settings'
require 'rox/server/client/server_properties'
require 'rox/core/properties/device_property'
require 'rox/core/properties/custom_property'
require 'rox/core/properties/custom_property_type'
require 'rox/core/consts/property_type'

module Rox
  module Core
    # Consolidates creation of all the internal properties
    class PropertyFactory
      def initialize(server_properties)
        @server_properties = server_properties
      end

      def platform
        property = Rox::Core::PropertyType::PLATFORM
        type = Rox::Core::CustomPropertyType::STRING

        Rox::Core::DeviceProperty.new(
          property.name,
          type,
          @server_properties.get(property)
        )
      end

      def app_release
        property = Rox::Core::PropertyType::APP_RELEASE
        type = Rox::Core::CustomPropertyType::SEMVER

        Rox::Core::DeviceProperty.new(
          property.name,
          type,
          @server_properties.get(property)
        )
      end

      def distinct_id
        property = Rox::Core::PropertyType::DISTINCT_ID
        type = Rox::Core::CustomPropertyType::STRING

        Rox::Core::DeviceProperty.new(
          property.name,
          type
        ) do |_|
          SecureRandom.uuid
        end
      end

      def internal_real_platform
        property = Rox::Core::PropertyType::PLATFORM
        type = Rox::Core::CustomPropertyType::STRING

        Rox::Core::DeviceProperty.new(
          'internal.realPlatform',
          type,
          @server_properties.get(property)
        )
      end

      def internal_custom_platform
        property = Rox::Core::PropertyType::PLATFORM
        type = Rox::Core::CustomPropertyType::STRING

        Rox::Core::DeviceProperty.new(
          'internal.customPlatform',
          type,
          @server_properties.get(property)
        )
      end

      def internal_app_key
        type = Rox::Core::CustomPropertyType::STRING

        Rox::Core::DeviceProperty.new(
          'internal.appKey',
          type,
          @server_properties.rollout_key
        )
      end

      def internal_distinct_id
        type = Rox::Core::CustomPropertyType::STRING

        Rox::Core::DeviceProperty.new(
          "internal.#{Rox::Core::PropertyType::DISTINCT_ID.name}",
          type
        ) do
          SecureRandom.uuid
        end
      end

      def internal_lib_version
        type = Rox::Core::CustomPropertyType::SEMVER

        Rox::Core::DeviceProperty.new(
          'internal.lib_version',
          type,
          @server_properties.lib_version
        )
      end

      def internal_api_version
        property = Rox::Core::PropertyType::API_VERSION
        type = Rox::Core::CustomPropertyType::SEMVER

        Rox::Core::DeviceProperty.new(
          'internal.api_version',
          type,
          property
        )
      end

      # rubocop:disable Metrics/MethodLength
      def all_properties
        [
          platform,
          app_release,
          distinct_id,
          internal_real_platform,
          internal_custom_platform,
          internal_app_key,
          internal_distinct_id,
          internal_lib_version,
          internal_api_version
        ]
      end
    end
  end
end
