module Rox
  module Core
    class Registerer
      def initialize(flag_repository)
        @flag_repository = flag_repository
        @namespaces = []
        @mutex = Mutex.new
      end

      def register_instance(container, namespace)
        raise ArgumentError, 'A namespace cannot be null' if namespace.nil?

        @mutex.synchronize do
          if @namespaces.include?(namespace)
            raise ArgumentError,
                  "A container with the given namespace (#{namespace}) has already been registered"
          end
        end

        @namespaces << namespace

        container.instance_variables.each do |attribute_name|
          begin
            value = container.instance_variable_get(attribute_name)
            if !value.nil? && value.is_a?(RoxString)
              var_name = attribute_name.to_s
            # removing the attribute starting @ (if [always] exists)
              var_name[0] = '' if var_name[0] == '@'
              @flag_repository.add_flag(value, namespace == '' ? var_name : "#{namespace}.#{var_name}")
            end
          rescue StandardError
            next
          end
        end
      end
    end
  end
end
