module Rox
  module Core
    class CustomPropertyRepository
      def initialize
        @custom_properties = {}
        @property_added_handlers = []
        @mutex = Mutex.new
        @handlers_mutex = Mutex.new
      end

      def add_custom_property(custom_property)
        return if custom_property.name.nil? || custom_property.name.empty?

        @mutex.synchronize do
          @custom_properties[custom_property.name] = custom_property
        end

        raise_property_added_event(custom_property)
      end

      def add_custom_property_if_not_exists(custom_property)
        return if custom_property.name.nil? || custom_property.name.empty?

        @mutex.synchronize do
          return if @custom_properties.include?(custom_property.name)
        end

        add_custom_property(custom_property)
      end

      def custom_property(name)
        @mutex.synchronize do
          return @custom_properties[name]
        end
      end

      def all_custom_properties
        @mutex.synchronize do
          return @custom_properties.values
        end
      end

      def register_property_added_handler(&block)
        @handlers_mutex.synchronize do
          @property_added_handlers << block
        end
      end

      def raise_property_added_event(custom_property)
        handlers = []
        @handlers_mutex.synchronize do
          handlers = @property_added_handlers.clone
        end

        handlers.each do |handler|
          handler.call(custom_property)
        end
      end
    end
  end
end
