module Rox
  module Core
    class FlagRepository
      def initialize
        @strings = {}
        @flag_added_handlers = []
        @mutex = Mutex.new
        @handlers_mutex = Mutex.new
      end

      def add_flag(string, name)
        string.name = name if string.name.nil? || string.name.empty?
        @mutex.synchronize do
          @strings[name] = string
        end
        raise_flag_added_event(string)
      end

      def flag(name)
        @mutex.synchronize do
          return @strings[name]
        end
      end

      def all_flags
        @mutex.synchronize do
          return @strings.values
        end
      end

      def register_flag_added_handler(&block)
        @handlers_mutex.synchronize do
          @flag_added_handlers << block
        end
      end

      def raise_flag_added_event(flag)
        handlers = []
        @handlers_mutex.synchronize do
          handlers = @flag_added_handlers.clone
        end

        handlers.each do |handler|
          handler.call(flag)
        end
      end
    end
  end
end
