module Rox
  module Core
    class RegularExpressionExtensions
      def initialize(parser)
        @parser = parser
      end

      def extend
        @parser.add_operator('match') do |_parser, stack, _context|
          text = stack.pop
          pattern = stack.pop
          flags = stack.pop
          unless text.is_a?(String) && pattern.is_a?(String) && flags.is_a?(String)
            raise ArgumentError,
                  'should be string'
          end

          options = 0
          flags.each_char do |flag|
            case flag
            when 'i'
              options |= Regexp::IGNORECASE
            when 'x'
              options |= Regexp::EXTENDED
            when 'm'
              options |= Regexp::MULTILINE
            end
          end

          matched = !Regexp.new(pattern, options).match(text).nil?
          stack.push(matched)
        end
      end
    end
  end
end
