module Rox
  module Core
    class StringTokenizer
      def initialize(text, delim, return_delims)
        @current_position = 0
        @new_position = -1
        @text = text
        @max_position = text.length
        @delimiters = delim
        @ret_delims = return_delims
      end

      def skip_delimiters(start_pos)
        raise ArgumentError, 'delimiters is nil' if @delimiters.nil?

        position = start_pos
        while !@ret_delims && position < @max_position
          c = @text[position]
          break unless delimiter?(c)

          position += 1
        end
        position
      end

      def scan_token(start_pos)
        position = start_pos
        while position < @max_position
          c = @text[position]
          break if delimiter?(c)

          position += 1
        end

        if @ret_delims && start_pos == position
          c = @text[position]
          position += 1 if delimiter?(c)
        end

        position
      end

      def delimiter?(char)
        @delimiters.include?(char)
      end

      def more_tokens?
        @new_position = skip_delimiters(@current_position)
        @new_position < @max_position
      end

      def next_token(delim = nil)
        delims_changed = false
        unless delim.nil?
          @delimiters = delim
          delims_changed = true
        end

        @current_position = @new_position >= 0 && !delims_changed ? @new_position : skip_delimiters(@current_position)
        @new_position = -1

        raise ArgumentError, 'invalid operation' if @current_position >= @max_position

        start = @current_position
        @current_position = scan_token(@current_position)
        @text[start...@current_position]
      end
    end
  end
end
