require 'rox/core/roxx/symbols'

module Rox
  module Core
    class TokenType
      attr_accessor :text, :pattern

      def self.from_token(token)
        unless token.nil?
          tested_token = token.downcase
          [TokenType::STRING, TokenType::NUMBER, TokenType::BOOLEAN, TokenType::UNDEFINED].each do |token_type|
            return token_type unless token_type.pattern.match(tested_token).nil?
          end
        end
        TokenType::NOT_A_TYPE
      end

      def initialize(text, pattern)
        @text = text
        @pattern = Regexp.new(pattern)
      end

      NOT_A_TYPE = TokenType.new('NOT_A_TYPE', '')
      STRING = TokenType.new(Symbols::ROXX_STRING_TYPE, '"((\\\\.)|[^\\\\"])*"')
      NUMBER = TokenType.new(Symbols::ROXX_NUMBER_TYPE, '[\\-]{0,1}\\d+[\\.]\\d+|[\\-]{0,1}\\d+')
      BOOLEAN = TokenType.new(Symbols::ROXX_BOOL_TYPE, "#{Symbols::ROXX_TRUE}|#{Symbols::ROXX_FALSE}")
      UNDEFINED = TokenType.new(Symbols::ROXX_UNDEFINED_TYPE, Symbols::ROXX_UNDEFINED)
    end
  end
end
