require 'rox/core/logging/logging'
require 'rox/server/logging/server_logger'

module Rox
  module Server
    class SelfManagedOptions
      attr_reader :server_url, :analytics_url

      def initialize(server_url:, analytics_url:)
        @server_url = server_url
        @analytics_url = analytics_url
      end
    end

    class RoxOptions
      attr_reader :dev_mode_key, :version, :fetch_interval,
                  :logger, :impression_handler,
                  :configuration_fetched_handler,
                  :roxy_url, :self_managed_options,
                  :dynamic_property_rule_handler

      def initialize(
        dev_mode_key: nil,
        version: nil,
        fetch_interval: nil,
        logger: nil,
        impression_handler: nil,
        configuration_fetched_handler: nil,
        roxy_url: nil,
        self_managed_options: nil,
        dynamic_property_rule_handler: nil
      )
        @dev_mode_key = dev_mode_key || 'stam'
        @version = version || '0.0'

        @fetch_interval = if fetch_interval.nil?
                            60
                          else
                            [fetch_interval, 30].max
                          end

        Rox::Core::Logging.logger = logger || ServerLogger.new

        @impression_handler = impression_handler
        @configuration_fetched_handler = configuration_fetched_handler
        @roxy_url = roxy_url
        @self_managed_options = self_managed_options
        @dynamic_property_rule_handler = dynamic_property_rule_handler || proc do |prop_name, context|
          context ? context[prop_name] : nil
        end
      end

      def self_managed?
        !@self_managed_options.nil?
      end
    end
  end
end
